Files
wehub-resource-sync 94057c3d3e
PR Test (NPU) / check-changes (push) Has been cancelled
PR Test (NPU) / pr-gate (push) Has been cancelled
PR Test (NPU) / set-image-config (push) Has been cancelled
PR Test (NPU) / stage-b-test-1-npu-a2 (0) (push) Has been cancelled
PR Test (NPU) / stage-b-test-1-npu-a2 (1) (push) Has been cancelled
PR Test (NPU) / stage-b-test-2-npu-a2 (0) (push) Has been cancelled
PR Test (NPU) / stage-b-test-2-npu-a2 (1) (push) Has been cancelled
PR Test (NPU) / stage-b-test-4-npu-a3 (push) Has been cancelled
PR Test (NPU) / stage-b-test-16-npu-a3 (push) Has been cancelled
PR Test (NPU) / multimodal-gen-test-1-npu-a3 (push) Has been cancelled
PR Test (NPU) / multimodal-gen-test-2-npu-a3 (push) Has been cancelled
PR Test (Arm64) / pr-gate (push) Has been cancelled
PR Test (Arm64) / check-changes (push) Has been cancelled
PR Test (Arm64) / build-test (push) Has been cancelled
PR Test (sgl-router) / gate (push) Has been cancelled
PR Test (sgl-router) / tier-1 — lint (push) Has been cancelled
PR Test (sgl-router) / tier-2 — build + test (push) Has been cancelled
PR Test (sgl-router) / tier-3 — docker (placeholder) (push) Has been cancelled
PR Test (sgl-router) / tier-3 — k8s integration (push) Has been cancelled
PR Test (sgl-router) / tier-3 — e2e (push) Has been cancelled
PR Test (sgl-router) / finish (push) Has been cancelled
PR Test (NPU) / single-node-poc (map[name:qwen3_6_27b_w8a8_1p_in64k_out1k_50ms runner:linux-aarch64-a3-2 test_case:test/registered/ascend/performance/qwen3_6_27b/test_npu_qwen3_6_27b_w8a8_1p_in64k_out1k_50ms.py test_type:perf]) (push) Has been cancelled
PR Test (NPU) / pr-test-npu-finish (push) Has been cancelled
PR Test (Xeon) / pr-gate (push) Has been cancelled
PR Test (Xeon) / check-changes (push) Has been cancelled
PR Test (Xeon) / build-test (, xeon-gnr, base-b-test-cpu) (push) Has been cancelled
PR Test (XPU) / check-changes (push) Has been cancelled
PR Test (XPU) / pr-gate (push) Has been cancelled
PR Test (XPU) / stage-a-test-1-gpu-xpu (push) Has been cancelled
PR Test (XPU) / wait-for-stage-a (push) Has been cancelled
PR Test (XPU) / stage-b-test-1-gpu-xpu (push) Has been cancelled
PR Test (XPU) / finish (push) Has been cancelled
CI Model Inventory / build-inventory (push) Has been cancelled
Lint / lint (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark Compilation Check (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark - Manual Policy (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark - Request Processing (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark Summary (push) Has been cancelled
PR Test (SMG) / build-wheel (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on windows (x86_64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on macos (x86_64 - auto) (push) Has been cancelled
PR Test (SMG) / python-unit-tests (push) Has been cancelled
PR Test (SMG) / unit-tests (push) Has been cancelled
PR Test (SMG) / benchmarks (push) Has been cancelled
PR Test (SMG) / chat-completions (push) Has been cancelled
PR Test (SMG) / chat-completions-4gpu (push) Has been cancelled
PR Test (SMG) / e2e (push) Has been cancelled
PR Test (SMG) / docker-build-test (push) Has been cancelled
PR Test (SMG) / k8s-integration (push) Has been cancelled
PR Test (SMG) / finish (push) Has been cancelled
PR Test (SMG) / summarize-benchmarks (push) Has been cancelled
Release SGLang Model Gateway Docker Image / publish (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on macos (aarch64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (aarch64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (x86_64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (aarch64 - musllinux_1_1) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (x86_64 - musllinux_1_1) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / Build SDist (push) Has been cancelled
Release SGLang Model Gateway to PyPI / Upload to PyPI (push) Has been cancelled
Release SGLang Kernels / build-cu129-matrix (aarch64, 12.9, 3.10, arm-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / build-cu129-matrix (x86_64, 12.9, 3.10, x64-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / release-cu129 (push) Has been cancelled
Release SGLang Kernels / build-cu130-matrix (aarch64, 13.0, 3.10, arm-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / build-cu130-matrix (x86_64, 13.0, 3.10, x64-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / release-cu130 (push) Has been cancelled
Release SGLang Kernels / build-rocm-matrix (3.10, 700) (push) Has been cancelled
Release SGLang Kernels / build-rocm-matrix (3.10, 720) (push) Has been cancelled
Release SGLang Kernels / release-rocm700 (push) Has been cancelled
Release SGLang Kernels / release-rocm720 (push) Has been cancelled
Release SGLang Kernels / build-musa43 (43, 3.10) (push) Has been cancelled
Release SGLang Kernels / release-musa43 (push) Has been cancelled
chore: import upstream snapshot with attribution
2026-07-13 12:38:16 +08:00

208 lines
6.2 KiB
Python

from typing import Optional, Tuple
import torch
import triton
import triton.language as tl
@triton.jit(do_not_specialize=["bs", "c128_cur_max_seq_len"])
def _init_compressed_attn_metadata_kernel(
seq_lens_ptr,
positions_ptr,
raw_out_loc_ptr,
page_table_ptr,
c4_out_loc_ptr,
c4_positions_ptr,
c4_seq_lens_raw_ptr,
c4_seq_lens_clamp1_ptr,
c128_out_loc_ptr,
c128_positions_ptr,
c128_seq_lens_raw_ptr,
c128_seq_lens_clamp1_ptr,
c128_page_indices_ptr,
bs,
max_pages,
c128_cur_max_seq_len,
c128_page_size: tl.constexpr,
BLOCK_SIZE: tl.constexpr,
COMPUTE_PAGE_INDICES: tl.constexpr,
):
batch_id = tl.program_id(0)
if batch_id >= bs:
return
seq_len = tl.load(seq_lens_ptr + batch_id)
position = tl.load(positions_ptr + batch_id)
raw_out_loc = tl.load(raw_out_loc_ptr + batch_id)
c4_should_compress = (seq_len % 4) == 0
c4_out_loc = tl.where(c4_should_compress, raw_out_loc // 4, 0)
c4_positions = position & (~3)
c4_seq_lens_raw = seq_len // 4
c4_seq_lens_clamp1 = tl.maximum(c4_seq_lens_raw, 1)
tl.store(c4_out_loc_ptr + batch_id, c4_out_loc)
tl.store(c4_positions_ptr + batch_id, c4_positions)
tl.store(c4_seq_lens_raw_ptr + batch_id, c4_seq_lens_raw)
tl.store(c4_seq_lens_clamp1_ptr + batch_id, c4_seq_lens_clamp1)
c128_should_compress = (seq_len % 128) == 0
c128_out_loc = tl.where(c128_should_compress, raw_out_loc // 128, 0)
c128_positions = position & (~127)
c128_seq_lens_raw = seq_len // 128
c128_seq_lens_clamp1 = tl.maximum(c128_seq_lens_raw, 1)
tl.store(c128_out_loc_ptr + batch_id, c128_out_loc)
tl.store(c128_positions_ptr + batch_id, c128_positions)
tl.store(c128_seq_lens_raw_ptr + batch_id, c128_seq_lens_raw)
tl.store(c128_seq_lens_clamp1_ptr + batch_id, c128_seq_lens_clamp1)
if COMPUTE_PAGE_INDICES:
page_indices_base = batch_id * c128_cur_max_seq_len
for block_start in tl.range(0, c128_cur_max_seq_len, BLOCK_SIZE):
offsets = block_start + tl.arange(0, BLOCK_SIZE)
mask = offsets < c128_cur_max_seq_len
page_idx = offsets // c128_page_size
offset_in_page = offsets % c128_page_size
page_mask = mask & (page_idx < max_pages)
page_table_vals = tl.load(
page_table_ptr + batch_id * max_pages + page_idx,
mask=page_mask,
other=0,
)
c_page_indices_vals = page_table_vals * c128_page_size + offset_in_page
valid_mask = offsets < c128_seq_lens_raw
c_page_indices_vals = tl.where(valid_mask, c_page_indices_vals, -1)
tl.store(
c128_page_indices_ptr + page_indices_base + offsets,
c_page_indices_vals,
mask=mask,
)
def _init_compressed_attn_metadata_triton(
seq_lens: torch.Tensor,
positions: torch.Tensor,
raw_out_loc: torch.Tensor,
page_table: Optional[torch.Tensor] = None,
page_size: int = 0,
compute_page_indices: bool = True,
) -> Tuple[
torch.Tensor,
torch.Tensor,
torch.Tensor,
torch.Tensor,
torch.Tensor,
torch.Tensor,
torch.Tensor,
torch.Tensor,
Optional[torch.Tensor],
]:
bs = seq_lens.shape[0]
device = seq_lens.device
c4_out_loc = torch.empty(bs, dtype=torch.int64, device=device)
c4_positions = torch.empty(bs, dtype=torch.int32, device=device)
c4_seq_lens_raw = torch.empty(bs, dtype=torch.int32, device=device)
c4_seq_lens_clamp1 = torch.empty(bs, dtype=torch.int32, device=device)
c128_out_loc = torch.empty(bs, dtype=torch.int64, device=device)
c128_positions = torch.empty(bs, dtype=torch.int32, device=device)
c128_seq_lens_raw = torch.empty(bs, dtype=torch.int32, device=device)
c128_seq_lens_clamp1 = torch.empty(bs, dtype=torch.int32, device=device)
if compute_page_indices:
assert (
page_table is not None
), "page_table required when compute_page_indices=True"
assert (
page_size >= 128 and page_size % 128 == 0
), "page_size must be a multiple of 128 when compute_page_indices=True"
max_pages = page_table.shape[1]
c128_page_size = page_size // 128
c128_cur_max_seq_len = c128_page_size * max_pages
c128_page_indices = torch.empty(
bs, c128_cur_max_seq_len, dtype=torch.int32, device=device
)
BLOCK_SIZE = triton.next_power_of_2(max(c128_page_size, 64))
else:
max_pages = 0
c128_page_size = 1
c128_cur_max_seq_len = 0
c128_page_indices = None
BLOCK_SIZE = 64
if page_table is None:
page_table = torch.empty(0, dtype=torch.int32, device=device)
grid = (bs,)
_init_compressed_attn_metadata_kernel[grid](
seq_lens,
positions,
raw_out_loc,
page_table,
c4_out_loc,
c4_positions,
c4_seq_lens_raw,
c4_seq_lens_clamp1,
c128_out_loc,
c128_positions,
c128_seq_lens_raw,
c128_seq_lens_clamp1,
(
c128_page_indices
if c128_page_indices is not None
else torch.empty(0, dtype=torch.int32, device=device)
),
bs,
max_pages,
c128_cur_max_seq_len,
c128_page_size,
BLOCK_SIZE,
compute_page_indices,
)
return (
c4_out_loc,
c4_positions,
c4_seq_lens_raw,
c4_seq_lens_clamp1,
c128_out_loc,
c128_positions,
c128_seq_lens_raw,
c128_seq_lens_clamp1,
c128_page_indices,
)
def init_compression_metadata(
seq_lens: torch.Tensor,
positions: torch.Tensor,
raw_out_loc: torch.Tensor,
page_table: Optional[torch.Tensor] = None,
page_size: int = 0,
compute_page_indices: bool = True,
) -> Tuple[
torch.Tensor,
torch.Tensor,
torch.Tensor,
torch.Tensor,
torch.Tensor,
torch.Tensor,
torch.Tensor,
torch.Tensor,
Optional[torch.Tensor],
]:
return _init_compressed_attn_metadata_triton(
seq_lens,
positions,
raw_out_loc,
page_table,
page_size,
compute_page_indices,
)